/*
back - back projector 
*/

static char rcsid[] = "$Id: back.c,v 1.8 2008/03/07 18:54:45 carl Exp $";

#include <math.h>
#include <parse.h>
#include "ct.h"

static int rmax=4095;		/*ramtek maximum*/
static int offset=0;		/*ramtek offset*/
static float pscale=1.0;	/*picture scale*/
static int print=1;		/*flag for printing program info*/
static int algo;		/*0=parallel 1=fan-equispace 2=fan-equiangle */
static int line=0;		/*1=line reconstruction*/
static int nx=128;		/*size reconstruction grid - x*/
static int ny=128;		/*size reconstruction grid - y*/
static float theta=0.0;		/*rotation*/
static float xo=0.0;		/*x center of grid*/
static float yo=0.0;		/*y center of grid*/
static float xfov;		/* x - field of view */
static float yfov;		/* x - field of view */
static float pixel;		/* pixel size; override fov */
static float yy1s=0.0;		/*y-coordinate in line mode*/
static char *ofn="pic";		/*output file name*/
static char *ifn="fproj.da";	/*input file name*/
static int noise=0;		/*1=simulate noise variance */
static MODE imode=0;		/* interplotaion mode (from header) */
static HU hu;			/*header for input and output files*/
static char *label="back";	/* annotation */
static int daout;		/* 1=output da-file instead of mi-file */
static float mu_water = 0.188;	/* mu of water */
static float ct_water = 1024;	/* ct # of water */

struct PARSE_OPTION_TABLE parse_table[] = {
	{"y",PARSE_FLOAT,(PARSE_ARG)&yy1s,"position of line in line mode"},
	{"y",PARSE_SET,(PARSE_ARG)&line},
	{"xo",PARSE_FLOAT,(PARSE_ARG)&xo,"x grid center"},
	{"yo",PARSE_FLOAT,(PARSE_ARG)&yo,"y grid center"},
	{"fov",PARSE_FLOAT,(PARSE_ARG)&xfov,"field of view"},
	{"fov",PARSE_FLOAT,(PARSE_ARG)&yfov},
	{"xfov",PARSE_FLOAT,(PARSE_ARG)&xfov,"x field of view"},
	{"yfov",PARSE_FLOAT,(PARSE_ARG)&yfov,"y field of view"},
	{"npic",PARSE_INT,(PARSE_ARG)&nx,"size of row/col of image"},
	{"npic",PARSE_INT,(PARSE_ARG)&ny},
	{"nx",PARSE_INT,(PARSE_ARG)&nx,"x size of image"},
	{"ny",PARSE_INT,(PARSE_ARG)&ny,"y size of image"},
	{"pixel",PARSE_FLOAT,(PARSE_ARG)&pixel,"pixel size"},
	{"scale",PARSE_FLOAT,(PARSE_ARG)&pscale,"image scale factor"},
	{"theta",PARSE_FLOAT,(PARSE_ARG)&theta,"image rotation (degrees)"},
	{"mu_water",PARSE_FLOAT,(PARSE_ARG)&mu_water,"attenuation of water"},
	{"ct_water",PARSE_FLOAT,(PARSE_ARG)&ct_water,"ct # of water"},
	{"if",PARSE_CHAR,(PARSE_ARG)&ifn,"input filename [da]"},
	{"=",PARSE_CHAR,(PARSE_ARG)&ifn,"if"},
	{"of",PARSE_CHAR,(PARSE_ARG)&ofn,"output filename [mi|da]"},
	{"offset",PARSE_INT,(PARSE_ARG)&offset,"image offset"},
	{"label",PARSE_CHAR,(PARSE_ARG)&label,"annotation"},
	{"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
	{0,0,0}
};

struct PARSE_FLAG_TABLE parse_flag_table[] = {
	{'l',&line,PARSE_FLAG_SET,"line reconstruction"},
	{'n',&noise,PARSE_FLAG_SET,"simulate noise variance"},
	{'d',&daout,PARSE_FLAG_SET,"write as da-file"},
	{'p',&print,PARSE_FLAG_CLR,"don't print status information"},
	{'?',(int *)(parse_help_basic),0,0},
	{0,0,0,0}
};

int
main(int argc,char **argv)
{
	int pp;			/* pixel pointer used in scale & offset of image */
	int irays;		/* number of input samples (rays) */
	int ii1;		/* 1 or ny */
	float ceni;		/* center sample on input */
	float cen;		/* center sample after interpolation */
	float pi;		/* banana */
	float convert;		/* radians to degrees */
	float chi;		/* fan angle */
	float r1;		/* source center distance in detector units */
	float cost,sint;	/* cos/sin of projection angle */
	float loc;		/* locator for inner loops */
	int locp;		/* integer version of loc */
	float t1,t1s,s1s,s1;	/* x,y rotated coordinates */
	float ssp,ss;		/* locators for fan beam beam projections */
	FILE *iu;		/* file pointe for reading */
	int iblk,nblk;		/* block pointers/counters for reading */
	int iii;		/* looper over views */
	int i,j;		/* loopers */
	float *picp,*pic;	/* picture pointers */
	short *picmi,*picmip; 	/* picture pointers, mi-version */
	float *sp,*dp;		/* projection pointers */
	int views;		/*number of projections - from header*/
	int rays;		/*number of samples per projection -header*/
	float *sproj,*proj;	/*projection*/
	float radius;		/*radius of scan circle*/
	float sc;		/*distance from source to center*/
	float alpha;		/* starting angle from header */
	float dtheta;		/* bump between projections from header */
	float ppscale;		/* image scale from filt*/
	float slide;		/*z-slide [mm] per 360 rotation*/
	float wdiso;		/* width of detector at iso-center */
	float *wgh;		/*weight function for equi-angle fan beam */
	float da;		/* angle between detectors */
	int nrot;		/*# 360 degree rotations */
	float px,py;		/* pixel sizes */
	float dtdx,dtdy;	/* loc derivatives */
	float dsdx,dsdy;	/* perpendicular loc derivatives */
/*
	parse command line and general initialization
*/
	parse_all_basic(".back","BACK",argc,argv);

	pi = M_PI;
/*
	open files
*/
	ifn = parse_suffix_malloc(ifn,"da");
	if(line == 0){
		ofn = parse_suffix_malloc(ofn,daout? "da" : "mi");
	}else{
		ofn = parse_suffix_malloc(ofn,"da");
	}
	iu = parse_fopen(ifn,"r");
/*
	get header information
*/
	ct_read_header(iu,&hu);

	rays = hu.heads[CT_N];
	views = hu.heads[CT_K];
	algo = hu.heads[CT_ALGO];
	imode = hu.heads[CT_MODE];
	nrot = hu.heads[CT_ROT];		

	radius = hu.headf[CT_R];
	sc = hu.headf[CT_RSC];
	alpha = hu.headf[CT_ALPHA];
	dtheta = hu.headf[CT_DBETA];
	ppscale = hu.headf[CT_SCALE];
	slide = hu.headf[CT_SLIDE];
	ceni = cen = hu.headf[CT_CEN];
	wdiso = hu.headf[CT_WDISO];

	iblk = 1;
	nblk = (rays + 127) / 128;

	if(!parse_opexist("fov") && 
	   !parse_opexist("xfov") && 
	   !parse_opexist("yfov")){
		xfov = yfov = 2.0 * radius;
	}

	if(parse_opexist("pixel")){
		xfov = nx * pixel;
		yfov = ny * pixel;
	}
	px = xfov / nx;
	py = yfov / ny;
/*
	check parameters
*/
        parse_pcheck(nrot,1,1024,"back: bad rotations in header");
	parse_pcheck(imode,0,MAX_MODE,"back: illegal projection mode");
	parse_pcheck(algo,0,2,"back: illegal algorithm");
	if(rays < 1) parse_pri("*back: illegal ray count: ",rays);
	if(views < 1) parse_pri("*back: illegal view count: ",views);
	if(wdiso <= 0) parse_prf("*back: illegal detector width: ",wdiso);
	if(radius <= 0) parse_prf("*back: illegal radius: ",radius);
/*
	set up miscellaneous parameters
*/
	if(algo != CT_PAR){		/* fan angle */
		chi = 2 * atan2(radius,sqrt(sc*sc-radius*radius)) * 180 / pi;
	}
/*
	print program status
*/
	if(print  ==  1){
		parse_wtext("******************** back ********************");
		parse_pdt("---start time");
		parse_rcsid_print(rcsid);
		parse_pri("n =",rays);
		parse_pri("k (total) =",views);
                parse_pri("nrot =",nrot);
		parse_prf("dtheta =",dtheta);
                parse_prf("wd (iso) =",wdiso);
		parse_prf("scan radius =",radius);
		parse_pri("nx =",nx);
		parse_pri("ny =",ny);
		parse_prf("xfov =",xfov);
		parse_prf("yfov =",yfov);
		parse_prf("pixel-x =",px);
		parse_prf("pixel-y =",py);
		parse_prf("xo =",xo);
		parse_prf("yo =",yo);
		if(line == 1) parse_prf("line reconstruction at y =",yy1s);
		parse_prf("theta =",theta);
		parse_prf("alpha =",alpha);
		parse_prf("scale =",pscale);
		parse_prf("mu water =",mu_water);
		parse_prf("ct# water =",ct_water);
		parse_prf("center detector =",ceni);
		parse_prf("filter scale =",ppscale);
		parse_pri("offset =",offset);
                parse_prt("noise variance",noise);
		parse_prs("label:",label);
		ct_print_algo(algo);
		if(algo != CT_PAR){
			parse_prf("rsc =",sc);
			parse_prf("fan angle (degrees) =",chi);
		}
		parse_prs("input file:",ifn);
		parse_prs("output file:",ofn);
		if(!line) parse_prt("da-format",daout);

		parse_prf("slide (mm) =",slide);
		ct_print_mode(imode);
	}
/*
	init reconstruction parameters
*/
	convert = pi / 180.0;
	irays = rays;

	ceni = cen;
	cen = cen + irays/2;

	if(algo == CT_PAR){
		; /* nothing to set up */
	}else if(algo == CT_SPACE){
		r1 = sc / wdiso;
		if(!noise) pscale *= sc * sc;
	}else{
		r1 = 1.0 / atan(wdiso/sc);
	}

	xo += -xfov / 2.0 + px/2;
	if(line == 0){
		yo += yfov / 2.0 - py/2;
		ii1 = ny;
	}else{
		ii1 = 1;
	}
/*
	get arrays
*/
	wgh = parse_malloc_float(rays);
	proj = parse_malloc_float(BLOCK_FLOAT(rays)*512/sizeof(float));
	sproj = parse_malloc_float(rays*2);
	pic = parse_malloc_float(ii1*nx);
	parse_zrbuf(pic,ii1*nx);
	parse_zrbuf(sproj,rays*2);
/*
	set up cosine squared of angle for equi-angle fan
*/
        if(algo == CT_ANGLE){
		da = atan(wdiso/sc);
		for(i=0;i<irays;i++){
                        wgh[i] = pow(cos((i-ceni) * da),2.0);
		}
        }
/*
	loop through projections
*/
	theta = theta * convert + alpha;

	for(iii=0;iii<views;iii++){
/*
	get angualar information
*/
	cost = cos(theta);
	sint = sin(theta);
/*
	read projection
*/
	parse_rblock(iu,iblk,(char *)proj,nblk);
	parse_from_can_floatv(proj,irays);
	iblk += nblk;
/*
	weight projections
*/
	if(algo == CT_ANGLE){
		for(i=0;i<irays;i++){
			proj[i] *= wgh[i];
		}
	}
/*
	square projection for noise estimation
*/
	if(noise) for(i=0;i<irays;i++) proj[i] = pow(proj[i],2.0);
/*
	pad with zeros to avoid boundary checks in inner loops
*/
	sp = sproj + irays/2;
	dp = proj;
	for(i=0;i<irays;i++) *sp++ = *dp++;
/*
	parallel projections
*/
	if(algo == CT_PAR){
		dtdx = cost * px / wdiso;
		dtdy = sint * py / wdiso;
		t1s = (xo*cost + yo*sint) / wdiso + cen;
		picp = pic;

		for(i=0;i<ii1;i++){
			t1 = t1s;
			for(j=0;j<nx;j++){
				locp = t1;
				*picp++ += sproj[locp] + 
					(t1-locp)*(sproj[locp+1] - sproj[locp]);
				t1 += dtdx;
			}
			t1s -= dtdy;
		}
	}
/*
	equal-space fan-beam projections
*/
	if(algo == CT_SPACE){
		dtdx = px * cost * r1;
		dtdy = py * sint * r1;
		dsdx = px * sint;
		dsdy = py * cost;

		t1s = (xo*cost + yo*sint) * r1;
		s1s = sc - (-xo*sint + yo*cost);

		picp = pic;

		for(i=0;i<ii1;i++){
			t1 = t1s;
			s1 = s1s;
			for(j=0;j<nx;j++){
				ssp = ss = 1.0 / s1;
				if(noise) ssp = pow(ss*sc,2.0);
				locp = loc = t1 * ss + cen;
				*picp++ += (sproj[locp] + 
					(loc-locp)*(sproj[locp+1] - sproj[locp]))
					* ssp * ssp;
				t1 += dtdx;
				s1 += dsdx;
			}
			t1s -= dtdy;
			s1s += dsdy;
		}
	}
/*
	equal-angle fan-beam projections
*/
	if(algo == CT_ANGLE){
		dtdx = px * cost;
		dtdy = py * sint;
		dsdx = px * sint;
		dsdy = py * cost;

		t1s = xo*cost + yo*sint;
		s1s = sc - (-xo*sint + yo*cost);

		picp = pic;

		for(i=0;i<ii1;i++){
			t1 = t1s;
			s1 = s1s;
			for(j=0;j<nx;j++){
				ssp = 1.0 / s1;
				locp = loc = atan(t1*ssp) * r1 + cen;
				*picp++ += (sproj[locp] + 
					(loc-locp)*(sproj[locp+1] - sproj[locp]))
					* ssp * ssp;
				t1 += dtdx;
				s1 += dsdx;
			}
			t1s -= dtdy;
			s1s += dsdy;
		}
	}
/*
	end projection loop
*/
	theta += dtheta;
	}
/*
	scale and output picture

	see NOTES for more info 
*/
	pscale *= ppscale * 10.0 * ct_water / mu_water / pi ;
	if(algo == CT_ANGLE) pscale *= sc *sc ;

	if(line == 1){
		if(noise){
			for(j=0;j<nx;j++){
				pic[j] = sqrt(pic[j]);
			}
		}

		for(picp=pic,j=0;j<nx;picp++,j++){
			*picp = *picp * pscale + offset;
		}
		parse_wda(pic,nx,1,ofn);
	}else{
		if(daout == 0){
			picmip = picmi = (short *)pic;
			for(picp=pic,i=0;i<ii1;i++){
				for(j=0;j<nx;j++){
					pp = *picp++ * pscale + 0.5 + offset;
					if(pp < 0) pp = 0;
					if(pp > rmax) pp = rmax;
					*picmip++ = pp;
				}
			}
			parse_wmi_l(ofn,picmi,nx,ny,label);
		}else{
			for(picp=pic,i=0;i<ii1;i++){
				for(j=0;j<nx;j++){
					*picp = *picp * pscale + 0.5 + offset;
					picp++;
				}
			}
			parse_wda(pic,nx,ny,ofn);
		}
	}
/*
	clean up and exit
*/
	if(print == 1)parse_pdt("---stop time");
	exit(0);
}
