/* Convert binary shorts to mi-file */

#include <parse.h>
#include <math.h>
#include "ct.h"
#include <string.h>

/* default option values : */
static char *ifn="infile.bin"; 
static char *ofn="outfile.mi";
static int offset = 0;
static  int    ncol = 512;
static  int    nrow = 512; 
static int bigendian_flag=1;  /* "input data is big-endian" flag */
static int header = 0;         /* header size in bytes */
static int footer = 0;         /* footer size in bytes */
static int skip = 0;		/* number of images to skip */ 
 
static char rcsid[] = "$Id: bin2mi.c,v 1.3 2008/03/07 18:55:50 carl Exp $";


/* Defining command line options' structure */
struct PARSE_OPTION_TABLE parse_table[] = {
  {"if",PARSE_CHAR,(PARSE_ARG)&ifn,"Name of the input file"},   
  {"=",PARSE_CHAR,(PARSE_ARG)&ifn,"Name of the input file"},   
  {"of",PARSE_CHAR,(PARSE_ARG)&ofn,"Name of the output file"},   
  {"ncol",PARSE_INT,(PARSE_ARG)&ncol,"No. of columns"},
  {"nrow",PARSE_INT,(PARSE_ARG)&nrow,"No. of rows"},
  {"offset",PARSE_INT,(PARSE_ARG)&offset,"Offset value to be subtracted"},   
  {"header",PARSE_INT,(PARSE_ARG)&header,"header size in bytes"},
  {"footer",PARSE_INT,(PARSE_ARG)&footer,"footer size in bytes"},
  {"skip",PARSE_INT,(PARSE_ARG)&skip,"number of images to skip"},
  {"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
  {0,0,0,0}
};

/* Command line flag structure */
struct PARSE_FLAG_TABLE parse_flag_table[] = {
  {'B',&bigendian_flag,PARSE_FLAG_SET,"Big-endian input (default)"},
  {'L',&bigendian_flag,PARSE_FLAG_CLR,"Litlle-endian input"},
  {'?',(int *)(parse_help_basic),0,0},
  {0,0,0,0}
};


  
int main(int argc, char* argv[])  {

  int i;
  short *dat;   /* data */
  char *head; /* header */
  FILE *fin;

  /* Reading command line parameters */
  parse_all_basic(NULL,NULL,argc,argv); 
  parse_wtext("-------- bin2mi  --------");
  parse_rcsid_print(rcsid);

  dat = parse_malloc_short(ncol*nrow);
  head = parse_malloc_char(header);

  ofn = parse_suffix_malloc(ofn,"mi");   
  parse_prs("ofn: ",ofn);
  parse_pri("No. of columns: ", ncol);
  parse_pri("No. of rows: ",nrow);
  parse_pri("Offset to be subtracted: ", offset);
  parse_pri("Images to skip: ", skip);

  parse_prs("Reading data from file ",ifn);
  fin = parse_fopen(ifn, "r");

 /* Read the header */
  if (header>0) {
    parse_fread(head,sizeof(char),header,fin); 
    /* rawheader is not used anywhere else - DR */
  }


  /* skip images */
  parse_fseek(fin,sizeof(short)*skip*ncol*nrow,"c");

  /* Read the data */
  parse_fread(dat, sizeof(short), ncol*nrow, fin);

  /* Convert to machine-native format */

  if (bigendian_flag == 1) {
    parse_from_can_shortv( dat, ncol*nrow );
    /* The raw data is in big-endian format (Sun) - DR */
  }
  else {
    parse_from_ican_shortv( dat, ncol*nrow);
    /* The raw data is in little-endian format (Linux, NT) - DR */
  }


  /*
    Subtract offset, if resulting value negative - zero out
  */
  

  for ( i = 0; i < ncol * nrow; i++) {
    
      
      dat [i] -= offset;

      if ( dat[i] < 0 ) dat[i] = 0;
      
    }
    
  parse_prs("Writing data into file ",ofn);
  parse_wmi(ofn, dat, ncol, nrow);

  parse_wtext("Program exited witout run-time errors.");
  return(0);
}

