/*
ct.h - common include file for ct code
*/

#ifndef CT_H
#define CT_H


//ADD THESE LINES HERE - GOVIND
#ifdef __cplusplus
extern "C"
{
#endif


static const char ct_h_rcsid[] = "$Id: ct.h,v 1.8 2008/03/07 18:55:50 carl Exp $";

/*
	interpolation modes
*/
typedef enum {
	HI,	/* 720 interpolated to 360 - HI */
	OS,	/* overscan - OS */
	HE,	/* extrapolative 360 - HE */
	HH,	/* helical halfscan - HH */
	HS,	/* segment - HS */
	US,	/* underscan - US */
	HH2,	/* helical halfscan, embodiment 2 - HH2 */
	FS,	/* pass straight through - FS */
	HSE,	/* segment with extra data - HSE */
	HA,	/* helical with asymmetric array - HA */
	HJ	/* helical jiang hsieh - HJ */
	/* *** be sure to update MAX_MODE *** */
} MODE;

#define MAX_MODE HJ

/*
	header for da-files
*/
typedef union {
	short heads[256];	/* integers in header */
	float headf[128];	/* floats in header */
} HU;

	/* offsets into heads */
#define CT_N	0		/* number detectors */
#define CT_K	1		/* number views */
#define CT_DA_TYPE 2		/* type of da-file, 0=float, 1=int, only int supported */

#define CT_ALGO	3		/* algorithm, 0=parallel 1=equispace 2=equiangle  */
#define CT_PAR 0		/* parallel projections */
#define CT_SPACE 1		/* equispace (fan) projections */
#define CT_ANGLE 2		/* equiangle (fan) projections */

#define CT_ROT	4		/* number of 2_pi rotations */
#define CT_MODE	5		/* interpolation mode */

	/* offsets into headf */
#define CT_R	4		/* radius of scan circle (mm) */
#define CT_RSC	5		/* source-to-center distance (mm) */
#define CT_ALPHA 6		/* angle of first view (radians) */
#define CT_DBETA 7		/* angle between views (radians) */
#define CT_SCALE 8		/* multiplicative scale */
#define CT_SLIDE 9		/* axial travel in 2_pi (mm) */
#define CT_CEN 10		/* center detector element */
#define CT_WDISO 11		/* detector width at isodenter (iso) */
#define CT_VSTART 12		/* starting view */
#define CT_CONEA 13		/* cone angle (radians) */
#define CT_WZISO 14		/* height of detector at isocenter */

	/* for header conversion */
#define CT_INT 6		/* number of integers to convert */
#define CT_FLOAT 4		/* number of floats not to convert */

/*
	prototypes
*/
void ct_print_algo(int);
void ct_print_mode(MODE);
void ct_read_header(FILE *,HU *);
void ct_write_header(FILE *,HU *);

//ADD THESE LINES HERE - GOVIND
#ifdef __cplusplus
}
#endif

#endif
