/*
dumphdr.c read da-file header and dump ct format fields to stdout
*/

#include <parse.h>
#include <math.h>
#include "ct.h"

static char rcsid[] = "$Id: dumphdr.c,v 1.3 2001/10/14 22:36:21 ccrawford Exp $";

static char *ifn="proj";	/* input filename */

struct PARSE_OPTION_TABLE parse_table[] = {
	{"if",PARSE_CHAR,(PARSE_ARG)&ifn,"input file"},
	{"=",PARSE_CHAR,(PARSE_ARG)&ifn,"if"},	
	{"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
	{0,0,0,0}
};

struct PARSE_FLAG_TABLE parse_flag_table[] = {
	{'?',(int *)(parse_help_basic),0,0},
	{0,0,0,0}
};


int main(int argc,char **argv)
{ 
	FILE *fp;
	HU hu;		/* header for input and output files*/
	float convert;	/* radians to degrees */
/*
	parse command line and other initialization
*/
	parse_all_basic(".dumphdr","DUMPHDR",argc,argv);

	convert = 180.0 / M_PI;
/* 
	read ct header 
*/
	ifn = parse_suffix_malloc(ifn,"da");
	fp = parse_fopen(ifn,"r");
	ct_read_header(fp,&hu);
/*
	do it
*/
	parse_wtext("******************* dumphdr *******************");
	parse_pdt("---start time");
	parse_rcsid_print(rcsid);
	parse_prs("input file:",ifn);
	parse_pri("rays =",hu.heads[CT_N]);
	parse_pri("views =",hu.heads[CT_K]);
	parse_pri("algo =",hu.heads[CT_ALGO]);
	ct_print_algo(hu.heads[CT_ALGO]);
	parse_pri("imode =",hu.heads[CT_MODE]);
	ct_print_mode(hu.heads[CT_MODE]);
	parse_pri("nrot =",hu.heads[CT_ROT]);
	parse_prf("radius =",hu.headf[CT_R]);
	parse_prf("sc =",hu.headf[CT_RSC]);
	parse_prf("alpha (radians) =",hu.headf[CT_ALPHA]);
	parse_prf("alpha (degrees) =",convert*hu.headf[CT_ALPHA]);
	parse_prf("dbeta (radians) =",hu.headf[CT_DBETA]);
	parse_prf("dbeta (degrees) =",convert*hu.headf[CT_DBETA]);
	parse_prf("ppscale =",hu.headf[CT_SCALE]);
	parse_prf("slide =",hu.headf[CT_SLIDE]);
	parse_prf("ceni = cen =",hu.headf[CT_CEN]);
	parse_prf("wdiso =",hu.headf[CT_WDISO]);
	parse_prf("vstart =",hu.headf[CT_VSTART]);
	parse_prf("conea (radians) =",hu.headf[CT_CONEA]);
	parse_prf("conea (degrees) =",convert*hu.headf[CT_CONEA]);
	parse_prf("wziso =",hu.headf[CT_WZISO]);
/*
	clean up and exit
*/
        parse_pdt("---stop time");
        exit(0);
}
