/* 
filt - filter tomographic projection data 
*/

static char rcsid[] = "$Id: filt.c,v 1.5 2008/03/07 18:55:50 carl Exp $";

#include	<math.h>
#include 	<parse.h>
#include 	"ct.h"

#define DEBUG
#ifdef DEBUG
#define PDT(x)	parse_pdt(x);
#endif

static char *ifn="proj.da";	/*input file name*/
static char *ofn="fproj.da";	/*output file name*/
static int han=0;		/*1=use hanning window*/
static int zp=0;		/*1=zero pad ifft to 2x length */
static float fbw=1.0;		/*fractional bw in window */
static int print=1;		/*flag for printing program info*/
static int rot=1;		/*which rotation to reconstruct*/
static int skip=0;		/*number of projections to skip */
static int bone=0;		/*1=bone detail*/

static float *pr1;		/*projection array*/
static float *pr2;		/*projection array*/
static float *wgh;		/*weight function for radial case*/
static float *wfilt;		/*mag(f) filter*/
static int noise=0;		/*1=simulate noise variance */
MODE imode;			/*interplotaion mode (from header) */
static int debug=0;		/* >0 debug mode */

static int custom=0;		/*1=custom kernel */
static float a2=0.6;		/* 2nd order coefficient for poly */
static float a3=-.2;		/* 3rd order coefficient for poly */
static float a4=0.0;		/* 4th order coefficient for poly */
static float beta=1.0;		/* user scale factor */
static float fstart=0.0;	/* start of lpf cutoff (lp/cm) */
static float fend=10.0;		/* end of lpf cutoff (lp/cm) */
static int cut=0;		/* lpf cut type 0=sinc 1=hanning */
static float gam=1.0;		/* power of lpf cutoff */
static float head_p=.16;	/* amplitude of head enhancement */
static float head_m=512;	/* power of head enhancment */ 
static int dohead=0;		/* 1= do head enhancement */
static int dopoly=0;		/* 1= do poly enhancement */
static float pixel=.01;		/* pixel size (mm) */
static float rho=1.0;		/* head enhancement frequency stretch */

struct PARSE_STRUCT parse_table[] = {
	{"if",PARSE_CHAR,(PARSE_ARG)&ifn,"input file"},
	{"=",PARSE_CHAR,(PARSE_ARG)&ifn,"if"},
	{"of",PARSE_CHAR,(PARSE_ARG)&ofn,"output file"},
	{"rot",PARSE_INT,(PARSE_ARG)&rot,"which rotation of data"},
	{"skip",PARSE_INT,(PARSE_ARG)&skip,"# view to skip"},
	{"fbw",PARSE_FLOAT,(PARSE_ARG)&fbw,"fractional bw"},

			/* stuff for custom kernels */
	{"a2",PARSE_FLOAT,(PARSE_ARG)&a2,"2nd order poly coefficient"},
	{"a2",PARSE_SET,(PARSE_ARG)&custom},
	{"a3",PARSE_FLOAT,(PARSE_ARG)&a3,"3rd order poly coefficient"},
	{"a3",PARSE_SET,(PARSE_ARG)&custom},
	{"a4",PARSE_FLOAT,(PARSE_ARG)&a4,"4th order poly coefficient"},
	{"a4",PARSE_SET,(PARSE_ARG)&custom},
	{"fstart",PARSE_FLOAT,(PARSE_ARG)&fstart,"start of lpf cutoff"},
	{"fstart",PARSE_SET,(PARSE_ARG)&custom},
	{"fend",PARSE_FLOAT,(PARSE_ARG)&fend,"end of lpf cutoff"},
	{"fend",PARSE_SET,(PARSE_ARG)&custom},
	{"gamma",PARSE_FLOAT,(PARSE_ARG)&gam,"cutoff power"},
	{"gamma",PARSE_SET,(PARSE_ARG)&custom},
	{"p",PARSE_FLOAT,(PARSE_ARG)&head_p,"amplitude of head enhancement"},
	{"p",PARSE_SET,(PARSE_ARG)&custom},
	{"m",PARSE_FLOAT,(PARSE_ARG)&head_m,"power of head enhancement"},
	{"m",PARSE_SET,(PARSE_ARG)&custom},
	{"rho",PARSE_FLOAT,(PARSE_ARG)&rho,"stretch of head enhancement"},
	{"rho",PARSE_SET,(PARSE_ARG)&custom},
	{"beta",PARSE_FLOAT,(PARSE_ARG)&beta,"user scale factor"},
	{"beta",PARSE_SET,(PARSE_ARG)&custom},
	{"pixel",PARSE_FLOAT,(PARSE_ARG)&pixel,"pixel size"},
	{"pixel",PARSE_SET,(PARSE_ARG)&custom},

	{"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
	{0,0,0}
};

struct PARSE_FLAG_TABLE parse_flag_table[] = {
	{'z',&zp,PARSE_FLAG_SET,"zero pad ifft"},
	{'h',&han,PARSE_FLAG_SET,"hanning window projections"},
	{'p',&print,PARSE_FLAG_CLR,"don't print status information"},
	{'b',&bone,PARSE_FLAG_SET,"bone detail"},
	{'n',&noise,PARSE_FLAG_SET,"simulate noise variance"},
	{'d',&debug,PARSE_FLAG_INC,"increase debug level"},

			/* stuff for custom kernels */
	{'g',&cut,PARSE_FLAG_SET,"hanning cutoff"},
	{'g',&custom,PARSE_FLAG_SET},
	{'c',&cut,PARSE_FLAG_CLR,"sinc cutoff"},
	{'c',&custom,PARSE_FLAG_SET},
	{'H',&dohead,PARSE_FLAG_SET,"do head enhancement"},
	{'H',&custom,PARSE_FLAG_SET},
	{'P',&dopoly,PARSE_FLAG_SET,"do poly enhancement"},
	{'P',&custom,PARSE_FLAG_SET},

	{'?',(int *)(parse_help_basic),0,0},
	{0,0,0,0}
};

static void
insertzero(float *p,int m)
{
	int i,j;

	for(i=0;i<m;i++){
		j = m - i - 1;
		p[2*j] = 2 * p[j];
		p[2*j+1] = 0;
	}
}

int
main(int argc,char **argv)
{
	FILE *iu,*ou;
	int oblk,iblk,nblki,nblko,i,ip,ii;
	int nfft;		/* size of fft */
	int nfft2,nfft21;	/* related to nfft */
	int nfftp,mp;
	float am;
	float x;
	int odd;
	int m;			/*related to size of fft*/

	int ni;			/* number of input samples per projection */
	int no;			/* number of output samples per projection */
	int n;			/*# of samples per projection during fft */
	int k;			/*number of projections */
	int algo;		/*0=parallel 1=fan-equispace 2=fan-equiangle */
	int nrot;		/*# 360 degree rotations */
	float r;		/*radius of scan circle*/
	float rsc;		/*source to center distance*/
	float alpha;		/* projection starting angle */
	float dbeta;		/* bump between projections */
	float scale;		/* scale factor for back*/
	float pi=M_PI;		/* in your face */
	float cen;		/* center detector */
	float ceno;		/* center output detector */
	float slide;		/*z-slide [mm] per 360 rotation*/
	float wdiso,wdisof,wdisoo;	/* width of detector at iso-center */
	float da;		/* angle subtended by one detector */
	HU hu; 			/*header for input and output files*/
	int userot;		/* 1= use rot from comamnd line */
	int kin;		/* input projections */

			/* stuff for custom kernels */
	int pl;			/* 1 = pixel limited */
	float fs;		/* fractional starting frequency */
	float fe;		/* fractional ending frequency */
	float f_max;		/* maximum spatial frequency (lp/cm) */
	int ns;			/* integer fractional starting frequency */
	int ne;			/* integer fractional ending frequency */
	int nw;			/* length of transition region */
	float f;		/* frequency sample */
	float g;		/* window sample */
	float pbw;		/* pixel bandwidth */
	float *lpf;		/* low pass filter */
	float *head;		/* head enhancement filter */
	float *poly;		/* polynomial filter */
	float *fa;		/* frequency array */
/*
	parse command line and misc initialization
*/
	parse_all_basic(".filt","FILT",argc,argv);

	userot = parse_opexist("rot");

        pi = M_PI;
/*
	open files
*/
	ifn = parse_suffix_malloc(ifn,"da");
	ofn = parse_suffix_malloc(ofn,"da");
	iu = parse_fopen(ifn,"r");
	ou = parse_fopen(ofn,"w");
/*
	get header information
*/
	ct_read_header(iu,&hu);

	n = hu.heads[CT_N];
	k = hu.heads[CT_K];
	algo = hu.heads[CT_ALGO];
	nrot = hu.heads[CT_ROT];		/* # rotations */
	imode = hu.heads[CT_MODE];		/* pass-through mode */

	r = hu.headf[CT_R];
	rsc = hu.headf[CT_RSC];
	alpha = hu.headf[CT_ALPHA];
	dbeta = hu.headf[CT_DBETA];
	scale = hu.headf[CT_SCALE];
	slide = hu.headf[CT_SLIDE];
	ceno = cen = hu.headf[CT_CEN];
	wdisof = wdisoo = wdiso = hu.headf[CT_WDISO];
/*
	set up for bone detail
*/
	ni = n;
	no = n;

	if(bone){
		wdisof /= 2;
		wdisoo /= 2;
		n *= 2;
		no *= 2;
		cen *= 2;
		ceno *= 2;
	}
/*
	handle skip
*/
	k = k / (skip + 1);
	dbeta *= (skip + 1);
/*
	set up fft size
*/
        am = log((float)(n*2))/log(2.0);
	m = am;
	if(am-m > 0.0)m = m + 1;
        nfftp = nfft = pow(2.0,(float) m);
        mp = m;
/*
	set up for zero padding
*/
	if(zp){
		wdisoo /= 2;
		ceno *= 2;
		nfftp = 2 * nfft;
		no *= 2;
		mp = m + 1;
		scale *= 2;
	}
/*
	check parameter validity and override header paraters
*/
        parse_pcheck(nrot,1,1024,"filt: bad rotations in header");
        if(userot) parse_pcheck(rot,1,nrot,"filt: bad rotation specification");
	parse_pcheck(imode,0,MAX_MODE,"filt: illegal projection mode");
/*
	set up parameters for custom kernel
*/
	nfft2 = nfft / 2;
	nfft21 = nfft2 + 1;
	f_max = 5.0 / wdisof; 	/* include qdo */
	if(!parse_opexist("fend")) fend = f_max;
	pbw = 10.0 / pixel;
	pl = fend > pbw;
/*
	setup and massage parameters
*/
        nblki = (ni + 127) / 128;
        nblko = (no + 127) / 128;
	oblk = 1;
	kin = k;

	if(userot){
		k /= nrot;
		nrot = 1;
		iblk = 1 + (rot-1)*nblki*k;
	}else{
		iblk = 1;
	}
/*
	print program status
*/
        if(print){
		parse_wtext("******************* filt *******************");
                parse_pdt("---start time");
		parse_rcsid_print(rcsid);
                parse_pri("n (input) =",ni);
                parse_pri("n (output) =",no);
                parse_pri("k (total) =",kin);
                parse_pri("k (rot) =",kin/nrot);
                parse_pri("k (out) =",k);
                parse_prf("r =",r);
                parse_prf("wd (iso) =",wdiso);
		parse_prf("alpha =",alpha);
		parse_prf("dbeta =",dbeta);
                parse_pri("nrot =",nrot);
                parse_prt("use rot",userot);
                if(userot) parse_pri("rot =",rot);
		parse_pri("skip =",skip);
		parse_pri("nfft =",nfft);
		ct_print_algo(algo);
                if(algo != CT_PAR) parse_prf("rs =",rsc);
		parse_prf("slide (mm) =",slide);
		parse_prf("center detector =",cen);
		ct_print_mode(imode);
		parse_prt("bone detail",bone);
                parse_prs("input file:",ifn);
                parse_prs("output file:",ofn);
                parse_prt("noise variance",noise);
                parse_prt("zero pad ifft",zp);

                if(custom == 0){
                	parse_wtext("window type: original");
			if(han == 1)parse_wtext("filter: hanning");
			parse_prf("fraction bw =",fbw);
		}else{
                	parse_wtext("window type: custom");
			parse_prf("a2 =",a2);
			parse_prf("a3 =",a3);
			parse_prf("a4 =",a4);
			parse_prf("fstart =",fstart);
			parse_prf("fend =",fend);
			parse_prf("gamma =",gam);
			parse_prf("beta =",beta);
			parse_prf("p =",head_p);
			parse_prf("m =",head_m);
			parse_prf("rho =",rho);
			if(cut == 0) parse_prs("lpf cut type:","sinc");
			if(cut == 1) parse_prs("lpf cut type:","hanning");
			parse_prt("poly enhancement",dopoly);
			parse_prt("head enhancement",dohead);
			parse_prt("pixel limited",pl);
			parse_prf("pixel =",pixel);
		}
        }
/*
	write header information
*/
        parse_zbuf(hu.heads,256);

	hu.heads[CT_N] = no;
	hu.heads[CT_K] = k;
	hu.heads[CT_ALGO] = algo;
	hu.heads[CT_ROT] = nrot;		
	hu.heads[CT_MODE] = imode;		

	hu.headf[CT_R] = r;
	hu.headf[CT_RSC] = rsc;
	hu.headf[CT_ALPHA] = alpha;
	hu.headf[CT_DBETA] = dbeta;
	hu.headf[CT_SCALE] = scale;
	hu.headf[CT_SLIDE] = slide;
	hu.headf[CT_CEN] = ceno;
	hu.headf[CT_WDISO] = wdisoo;

        ct_write_header(ou,&hu);
/*
	allocate arrays
*/
	pr1 = parse_malloc_float(nfftp);
	pr2 = parse_malloc_float(nfftp);
	wgh = parse_malloc_float(n);
	wfilt = parse_malloc_float(nfft);

	lpf = parse_malloc_float(nfft21);
	head = parse_malloc_float(nfft21);
	poly = parse_malloc_float(nfft21);
	fa = parse_malloc_float(nfft21);
/*
	set up algorithm specific information
*/
        if(algo == CT_SPACE){
		for(i=0;i<n;i++){
                        x = (i-cen) * wdisof;
                        wgh[i] = cos(atan(x/rsc));
		}
        }

	da = atan(wdisof/rsc);
        if(algo == CT_ANGLE){
		for(i=0;i<n;i++){
                        x = (i-cen) * da;
                        wgh[i] = cos(x);
		}
        }
/*
	set up kernel 

	see NOTES for more info 
*/
	for(i=0;i<nfft;i++){
		pr1[i] = 0.0;
		pr2[i] = 0.0;
	}

	if(algo == CT_PAR || algo == CT_SPACE){
		pr1[0] = .25 / wdisof * pi;
	}else{
		pr1[0] = .25 / da * pi / rsc;
	}

	for(i=1;i<nfft21;i += 2){
		x = i; 
		if(algo == CT_PAR || algo == CT_SPACE){
			pr1[i] = -1.0/(x*x*pi*wdisof);
		}else{
			pr1[i] = -da/(pi*sin(x*da)*sin(x*da))/rsc;
		}
		pr1[nfft-i] = pr1[i];
	}

	if(noise){
		for(i=0;i<nfft;i++){
			pr1[i] = pow(pr1[i],2.0);
		}
	}

	parse_fft(pr1,pr2,m,0);
/*
	set up "original" window
*/
        if(custom == 0){
		if(han == 1){
			wfilt[0] = 1.0;
			for(i=1;i<nfft21;i++){
				wfilt[i] = cos(i*pi/nfft/fbw);
			}
		}else{
			for(i=0;i<nfft;i++){
				wfilt[i] = 1.0;
			}
		}

		ip = nfft21 * fbw + 1;
		for(i=ip;i<nfft21;i++){
			wfilt[i] = 0;
		}
	}
/*
	custom kernel
*/
        	/* editor tag: peep */
	if(custom == 1){

			/* low pass filter */
		if(pl){
			fstart *= pbw/fend;
			fend = pbw;
		}

		fs = fstart / f_max;
		fe = fend / f_max;

		ns = 0.5 + fs * nfft21;
		ne = 0.5 + fe * nfft21;
		nw = ne - ns;

		for(i=0;i<nfft21;i++){
			if(i <= ns){
				lpf[i] = 1.0;
			}else if(i <= ns+nw){
				f = (i- ns + 1)*pi / (nw+1);
				if(cut == 0){
					g = sin(f)/f;
				}else{
					g = 0.5 + 0.5 * cos(f);
				}
				if(g < 0) g = 0;
				lpf[i] = pow(g,gam);
			}
		}
		if(debug){
			parse_wtext("lpf (custom) saved in: lpf.da");
			parse_wda(lpf,nfft21,1,"lpf");
		}

			/* head enhancement per pms */
		for(i=0;i<nfft21;i++){
			f = rho * pi * i / nfft;
			if(f > pi/2) f = pi/2;
			head[i] = 1.0 + head_p * ( 1.0 - pow(cos(f),head_m));
		}
		if(debug){
			parse_wtext("head (custom) saved in: head.da");
			parse_wda(head,nfft21,1,"head");
		}

			/* polynomial enhancement */
		for(i=0;i<nfft21;i++){
			f = (float) i / nfft21;
			poly[i] = 1.0 + f * f * (a2 + f * (a3 + a4*f));
		}
		if(debug){
			parse_wtext("poly (custom) saved in: poly.da");
			parse_wda(poly,nfft21,1,"poly");
		}

			/* generate window arrays */
		for(i=0;i<nfft21;i++){
			wfilt[i] = beta * lpf[i];
			if(dohead) wfilt[i] *= head[i];
			if(dopoly) wfilt[i] *= poly[i];
		}
	}
/*
	save window and frequencies
*/
	if(debug){
		parse_wtext("window saved in: window.da");
		parse_wda(wfilt,nfft21,1,"window");

			/* frequency array */
		for(i=0;i<nfft21;i++){
			fa[i] = i * f_max / nfft2;
		}
		parse_wtext("frequencies saved in (lp/cm): x.da");
		parse_wda(fa,nfft21,1,"x");
	}
/*
	apply window to kernel
*/
	for(i=0;i<nfft21;i++){
		wfilt[i] *= pr1[i];
	}

	for(i=1;i<nfft21;i++){
		wfilt[nfft-i] = wfilt[i];
	}
/*
	loop through projections
*/
	odd = k & 1;

	for(ii=0;ii<k;){
		parse_fread_block((char *)pr1,sizeof(float),ni,iu,iblk);
		parse_from_can_floatv(pr1,ni);
		if(bone) insertzero(pr1,ni);
		iblk += nblki * (skip + 1);
		if(odd == 1){
			for(i=0;i<n;i++)pr2[i] = 0.0;
			ii++;
		}else{
			parse_fread_block((char *)pr2,sizeof(float),ni,iu,iblk);
			parse_from_can_floatv(pr2,n);
			if(bone) insertzero(pr2,ni);
			iblk += nblki * (skip + 1);
			ii += 2;
		}
                if(algo != CT_PAR){
			for(i=0;i<n;i++){
                                pr1[i] *= wgh[i];
                                pr2[i] *= wgh[i];
			}

			if(noise){
				for(i=0;i<n;i++){
					pr1[i] = pow(pr1[i],2.0);
					pr2[i] = pow(pr2[i],2.0);
				}
			}
                }
		for(i=n;i<nfft;i++){
                        pr1[i] = 0.0;
                        pr2[i] = 0.0;
		}
                parse_fft(pr1,pr2,m,0);

		for(i=0;i<nfft;i++){
                        pr1[i] *= wfilt[i];
                        pr2[i] *= wfilt[i];
		}
                if(zp){
			for(i=nfft/2;i<nfft;i++){
				pr1[i+nfft] = pr1[i];
				pr2[i+nfft] = pr2[i];
			}
			for(i=nfft/2;i<nfft/2+nfft;i++){
				pr1[i] = pr2[i] = 0.0;
			}
		}

                parse_fft(pr1,pr2,mp,1);


		parse_to_can_floatv(pr1,no);
		parse_wblock(ou,oblk,(char *)pr1,nblko);
		oblk += nblko;
		if(!odd){
			parse_to_can_floatv(pr2,no);
			parse_wblock(ou,oblk,(char *)pr2,nblko);
			oblk += nblko;
		}else{
			odd = 0;
		}
	}
/*
	clean up and exit
*/
        if(print==1)parse_pdt("---stop time");
        exit(0);
}
