/*
mse - calculate mse of part of image(s)
*/

static char rcsid[] = "$Id: mse.c,v 1.4 2008/03/07 18:55:50 carl Exp $";

#include <parse.h>
#include <math.h>
#include "ct.h"

#define N 128			/* size of image */
#define M 40			/* rubout region */

#define N2 (N*N)

static char *ifn="pic.mi";	/* input filename */
static char *dfn;		/* difference filename */
static int print=1;		/* flag for printing program info */
static int noise=0;		/* 1=noise mode*/
static int baseline=1024;	/* ... of image (HU) */

struct PARSE_OPTION_TABLE parse_table[] = {
	{"if",PARSE_CHAR,(PARSE_ARG)&ifn,"input file"},
	{"=",PARSE_CHAR,(PARSE_ARG)&ifn,"if"},
	{"df",PARSE_CHAR,(PARSE_ARG)&dfn,"difference file"},
	{"baseline",PARSE_INT,(PARSE_ARG)&baseline,"image baseline"},
	{"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
	{0,0,0,0}
};

struct PARSE_FLAG_TABLE parse_flag_table[] = {
	{'n',&noise,PARSE_FLAG_SET,"noise measurements"},
	{'?',(int *)(parse_help_basic),0,0},
	{0,0,0,0}
};

int
main(int argc,char **argv)
{
	short int *a,*a1;	/* images */
	short int *ap,*a1p;	/* pointers to images */
	float x,y,z;		/* for mean variance calculations */
	int k,i,j;		/* loopers */
	int udf;		/* 1=difference mode */
/*
	parse command line and massage parameters
*/
	parse_all_basic(".mse","MSE",argc,argv);

	udf = parse_opexist("df");
/*
	open files
*/
	ifn = parse_suffix_malloc(ifn,"mi");
	a = parse_rmi(ifn,NULL,&i,&j);
	if(i != N || j != N)parse_err("mse: bad size:",ifn);

	if(udf){
		dfn = parse_suffix_malloc(dfn,"mi");
		a1 = parse_rmi(dfn,NULL,&i,&j);
		if(i != N || j != N)parse_err("mse: bad size:",dfn);
	}else{
		a1 = parse_malloc_short(N2);
		for(i=0;i<N2;i++) a1[i] = 0.0;
	}
/*
	print program status
*/
        if(print){
		parse_wtext("******************* mse *******************");
                parse_pdt("---start time");
		parse_rcsid_print(rcsid);
                parse_prs("input file:",ifn);
                if(udf) parse_prs("difference file:",dfn);
                parse_pri("nx =",N);
                parse_pri("ny =",N);
                parse_pri("baseline =",baseline);
        }
/*
	do it
*/
	if(noise == 0){
		for(i=0;i<M;i++){
			k = (N/2 - M/2 + i) * N + N/2 - M/2;
			ap = a + k;
			a1p = a1 + k;
			for(j=0;j<M;j++){
				*ap++ = *a1p++ = baseline;
			}
		}
	}

	for(x=i=0;i<N2;i++){
		if(udf){
			x += a[i] - a1[i];
		}else{
			x += a[i];
		}
	}
	x /= N2;
	for(y=i=0;i<N2;i++){
		if(udf){
			z = a[i] - a1[i] - x;
		}else{
			z = a[i] - x;
		}
		y += z*z;
	}
	y /= N2;
/*
	print results
*/
	printf("mse: ");
	if(udf){
		printf("%s %s: ",ifn,dfn);
	}else{
		printf("%s: ",ifn);
	}
	printf("mean/var/std %f %f %f\n",x,y,sqrt(y));
/*
	clean up and exit
*/
        if(print==1)parse_pdt("---stop time");
        return(0);
}
