/*
slice - form maximum projections and width measurements
*/

static char rcsid[] = "$Id: slice.c,v 1.4 2008/03/07 18:55:50 carl Exp $";

#include <parse.h>
#include <string.h>
#include <stdlib.h>
#include "ct.h"

#define N 128			/* size of image subset */

static char *ifn="pic.mi";	/* input file name */
static char *ofn;		/* output file name */
static char *ofnx="slice2x";	/* output file name for x-axis*/
static int print=1;		/* flag for printing program info */
static int debug=0;		/* print debug information */
static float pixel=.47;		/* pizel size (mm), .47 = 60/128 */
static int baseline=1024;	/* ... of image (HU) */
static int rot=0;		/* 1= rotated image, default column MIP */
static int xo,yo;		/* x- and y-center, default auto calc */
static int xos=0,yos=0;		/* xo= and yo= set */
static int half=0;		/* 1=use half measurements */
static int absp=1;		/* 1=MIP abs(pixels) */
static int npx=N;               /* x-size of image subset */
static int npy=N;               /* y-size of image subset */
static char *sfn="subset.mi";	/* image subset filename */
static char *bfn="base.da";	/* baseline filename */

struct PARSE_OPTION_TABLE parse_table[] = {
	{"if",PARSE_CHAR,(PARSE_ARG)&ifn,"input file"},
	{"=",PARSE_CHAR,(PARSE_ARG)&ifn,"if"},
	{"of",PARSE_CHAR,(PARSE_ARG)&ofn,"output file (default from ifn)"},
	{"xf",PARSE_CHAR,(PARSE_ARG)&ofnx,"x-axis output file"},
	{"pixel",PARSE_FLOAT,(PARSE_ARG)&pixel,"pixel size (mm)"},
	{"baseline",PARSE_INT,(PARSE_ARG)&baseline,"image baseline"},
	{"xo",PARSE_INT,(PARSE_ARG)&xo,"x-origin, pixels"},
	{"yo",PARSE_INT,(PARSE_ARG)&yo,"y-origin, pixels"},
	{"xn",PARSE_INT,(PARSE_ARG)&npx,"x-size of image subset"},
	{"yn",PARSE_INT,(PARSE_ARG)&npy,"y-size of image subset"},
	{"?",PARSE_PROC,(PARSE_ARG)parse_help_basic},
	{0,0,0,0}
};

struct PARSE_FLAG_TABLE parse_flag_table[] = {
	{'a',&absp,PARSE_FLAG_CLR,"don't abs(pixels)"},
	{'p',&print,PARSE_FLAG_CLR,"don't print program information"},
	{'d',&debug,PARSE_FLAG_INC,"enable debugging output"},
	{'r',&rot,PARSE_FLAG_SET,"rotated image"},

	{'h',&half,PARSE_FLAG_SET,"use half estimates"},
	{'?',(int *)(parse_help_basic),0,0},
	{0,0,0,0}
};

int
main(int argc,char **argv)
{
	short int *ip;		/* input image */
	int nx,ny;		/* size of input image */
	short *a;	        /* image subset */
	float del;		/* delta distance right - left */
	float *b;		/* image mip */
	float *xaxis;           /* x-vector for qplot */
	float x1,x2;		/* left and right baseline values */
	float lv,rv;		/* left/right intercepts */
	float max,maxs;		/* mip maximum value */
	int i,j,k,l,r,m;	/* loopers and int pointers */
	short int p;		/* pixel */
	int dx,dy;		/* increments for mip */
	float fw10m,fw20m,fw50m; /* should be obvious */
	float db;		/* delta between adjacent mip values */
	float xc,yc;            /* x- and y- corners of subset */
        float xyc;              /* x- or y- corner of subset */
	int npb;                /* length of output vectors (npx or npy) */
        int npc;                /* column length (npy or npx) */
/*
	parse command line and massage parameters
*/
	parse_all_basic(".slice","SLICE",argc,argv);

	xos = parse_opexist("xo");
	yos = parse_opexist("yo");

	ifn = parse_suffix_malloc(ifn,"mi");
	sfn = parse_suffix_malloc(sfn,"mi");
	bfn = parse_suffix_malloc(bfn,"da");

	if (rot) {
	        npb=npy;
		npc=npx;
	} 
	else {
                npb=npx;
		npc=npy;
	}

	a = parse_malloc_short(npx*npy);
	b = parse_malloc_float(npb);
	xaxis = parse_malloc_float(npb);

	if(!parse_opexist("of")){
		ofn = parse_malloc_char(strlen(ifn)+1);
		strcpy(ofn,ifn);
		parse_remove_suffix(ofn);
	}
	ofn = parse_suffix_malloc(ofn,"da");
/*
	open picture and get center npix*npix
*/
	ip = parse_rmi(ifn,NULL,&nx,&ny);

	if(xos == 0) xo = nx / 2;
	if(yos == 0) yo = ny / 2;
	xc = xo -= npx/2;
	yc = yo -= npy/2;

	if(xo < 0 || yo < 0 || xo+npx > nx || yo+npy > ny){
		parse_er("slice: subset not contained in input image");
	}

	k = yo * nx + xo;
	for(l=i=0;i<npy;i++){
		for(j=0;j<npx;j++){
			a[l++] = ip[k++];
		}
		k += nx - npx;
	}

	xo += npx/2;		/* restore for printing */
	yo += npy/2;

	if(debug) parse_wmi(sfn,a,npx,npy);
/*
	print program status
*/
        if(print){
		parse_wtext("******************* slice *******************");
                parse_pdt("---start time");
		parse_rcsid_print(rcsid);
                parse_prs("input file:",ifn);
                parse_prs("output file:",ofn);
                parse_pri("nx =",nx);
                parse_pri("ny =",ny);
                parse_pri("xo =",xo);
                parse_pri("yo =",yo);
                parse_pri("npx =",npx);
                parse_pri("npy =",npy);
                parse_prf("pixel =",pixel);
                parse_pri("baseline =",baseline);
                parse_prt("rotate",rot);
                parse_prt("abs(pixel)",absp);
		if(debug){
			parse_prs("subset:",sfn);
			parse_prs("baseline:",bfn);
		}
        }
/*
	form projection
*/
	if(rot){
		dx = npx;
		dy = 1;
		xyc = yc;
	}else{
		dx = 1;
		dy = npx;
		xyc = xc;
	}

	for(i=0;i<npb;i++){
		b[i] = 0;
		k = i*dx;
		xaxis[i] = (xyc + i)*pixel;
		for(j=0;j<npc;j++){
			p = a[k] - baseline;
			if(absp) p = labs(p);
			if(p > b[i]) b[i] = p;
			k += dy;
		}
	}

	parse_wda(b,npb,1,ofn);
	parse_wda(xaxis,npb,1,ofnx);
/*
	subtract baseline and find max
*/
	for(x1=x2=i=0;i<5;i++){
		x1 += b[i];
		x2 += b[npb-1-i];
	}
	x1 /= 5;
	x2 /= 5;
	for(j=max=i=0;i<npb;i++){
		b[i] -= x1 + i*(x2-x1)/npb;
		if(b[i] > max){
			max = b[i];
			j = i;
		}
	}

	if(debug){
		parse_wda(b,npb,1,bfn);
		printf("max/loc %f, %d\n",max,j); 
	}

/*
	get FWTM and FWHM
*/
	maxs = max; 

	for(m=0;m<3;m++){
		switch(m) {
			case 0: max = .5 * maxs; break;
			case 1: max = .2 * maxs; break;
			case 2: max = .1 * maxs; break;
		}

		for(i=j;i<npb;i++){
			if(b[i] < max) break;
		}
		if(debug)printf("right %d\n",i); 
		r = i;
		db = b[i-1] - b[i];
		if(db == 0)parse_er("slice: no intercept slope");
		rv = (b[i-1] - max) / db + i - 1;

		for(i=j;i>=0;i--){
			if(b[i] < max) break;
		}
		if(debug)printf("left %d\n",i); 

		l = i;
		db = b[i+1] - b[i];
		if(db == 0)parse_er("slice: no intercept slope");
		lv = (b[i+1] - max) / db + i;
		if(m && half){
			del = (rv - half) * 2;
		}else{
			del = rv - lv;
		}
		del *= pixel;

		switch(m) {
			case 0: fw50m = del; break;
			case 1: fw20m = del; break;
			case 2: fw10m = del; break;
		}

		if(!m && half) half = (lv + rv) / 2.0;
	}
/*
	report results
*/
	printf("slice: %s: FW50M %5.2f FW20M %5.2f FW20M %5.2f\n",ifn,
		fw50m,fw20m,fw10m);
/*
	clean up and exit
*/
	
	parse_free(a);
	parse_free(b);
	parse_free(xaxis);

        if(print==1)parse_pdt("---stop time");
        return(0);
}
